Package: ExtremeRisks
Version: 0.0.6
Date: 2026-05-08
Title: Extreme Risk Measures
Authors@R: c(person("Simone","Padoan", role = c("cre", "aut"), email="simone.padoan@unibocconi.it"),
	    person("Gilles", "Stupfler", role = "aut", email="gilles.stupfler@univ-angers.fr"),
	    person("Carlotta", "Pacifici", role = c("aut"), email="carlotta.pacifici@unibocconi.it"))
Author: Simone Padoan [cre, aut],
  Gilles Stupfler [aut],
  Carlotta Pacifici [aut]
Maintainer: Simone Padoan <simone.padoan@unibocconi.it>
Imports: evd, copula, mvtnorm, plot3D, tmvtnorm, pracma
Depends: R (>= 3.5.0)
Description: A set of procedures for estimating risks related to extreme events via risk measures such as Expectile, Value-at-Risk, etc. is provided. Estimation methods for univariate independent observations and temporal dependent observations are available. The methodology is extended to the case of independent multidimensional observations.  The statistical inference is performed through parametric and non-parametric estimators. Inferential procedures such as confidence intervals, confidence regions and hypothesis testing are obtained by exploiting the asymptotic theory. Adapts the methodologies derived in Padoan and Stupfler (2022) <doi:10.3150/21-BEJ1375>, Davison et al. (2023) <doi:10.1080/07350015.2022.2078332>, Daouia et al. (2018) <doi:10.1111/rssb.12254>, Drees (2000) <doi:10.1214/aoap/1019487617>, Drees (2003) <doi:10.3150/bj/1066223272>, de Haan and Ferreira (2006) <doi:10.1007/0-387-34471-3>, de Haan et al. (2016) <doi:10.1007/s00780-015-0287-6>, Padoan and Rizzelli (2024) <doi:10.3150/23-BEJ1668>, Daouia et al. (2024) <doi:10.3150/23-BEJ1632>.
License: GPL (>= 2)
URL: https://www.unibocconi.it/en/faculty/simone-padoan
NeedsCompilation: yes
Repository: CRAN
Repository/R-Forge/Project: extremerisks
Encoding: UTF-8
Packaged: 2026-05-11 15:01:16 UTC; padoan
Date/Publication: 2026-05-12 05:10:27 UTC
Built: R 4.6.0; ; 2026-05-12 05:45:37 UTC; unix
