export(MomTailIndex, MLTailIndex, HTailIndex, EBTailIndex, extQuantile, estExpectiles, predExpectiles, expectiles, rtimeseries, estMultiExpectiles, predMultiExpectiles, estExtLevel, QuantMES, ExpectMES, rbtimeseries, HypoTesting, MultiHTailIndex, extMultiQuantile, rmdata,
plotBayes, fitdGPD, estPOT, extBQuant, predDens, predDensx, predQuant, extBQuantx, scedastic.test, schedastic.test, cpost_stat, Bqgpd_c, Bqgpd_d, testTailHomo, quantF)

importFrom("stats", "df", "qf", "dnorm", "dt", "ecdf", "integrate", "optim",
           "pnorm", "pt", "quantile", "rnorm", "runif", "dunif", "uniroot",
           "var", "rpois", "dpois", "rbeta", "dbeta", "dnbinom", "pexp", "qt", 
	   "qchisq", "pbeta", "qnorm", "cov", "rnbinom", "rt", "rbinom", "acf",
	   "dgamma", "rexp", "rgamma", "sd")

importFrom("graphics", "par", "text", "legend", "lines", "points", "curve", "hist")

importFrom("utils", "combn", "setTxtProgressBar", "txtProgressBar")

importFrom("evd", "rbvevd", "abvnonpar", "amvnonpar", "pfrechet", "pgumbel", 
	   "pgev", "abvevd", "hbvevd", "rgpd", "rfrechet", "pgpd", "qgpd", "fpot")

importFrom("mvtnorm", "rmvt", "rmvnorm")

importFrom("tmvtnorm", "rtmvt", "rtmvnorm")

importFrom("plot3D", "scatter3D", "points3D")

importFrom("copula", "rCopula", "claytonCopula", "gumbelCopula", "frankCopula", "normalCopula")

importFrom("pracma", "sqrtm")

importFrom("stats", "median", "pchisq")
