\name{Bqgpd_c}
\alias{Bqgpd_c}
\title{Bayesian predictive quantile for generalized Pareto distribution}
\usage{
Bqgpd_c(p, post, lower, upper)
}
\arguments{
\item{p}{probability levels}

\item{post}{matrix of posterior samples}

\item{lower}{lower bound for the search for the quantile}

\item{upper}{upper bound for the region over which to search for the quantile}
}
\value{
a vector of quantiles of the same length as \code{p}
}
\description{
WARNING: this function will not be exported in a future version of the package.
}
\keyword{internal}
